/******************************************************************************
* ex_posterior.g
* plots surface of posterior for the simple example
* 
* upadted:  03/11/2005
******************************************************************************/

goto plotnow;

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/* some global variables
*/
rhohat = 0.7;
wt = 100;
wtp = 5 | 50;

/* Define Gridpoints
*/

alp_min = 0;
alp_max = 4;
alp_n = 501;
alp_grid = seqa( alp_min, (alp_max-alp_min)/(alp_n-1), alp_n);

rho_min = 0;
rho_max = 0.95;
rho_n = 501;
rho_grid = seqa( rho_min, (rho_max-rho_min)/(rho_n-1), rho_n);

phi_min = 0;
phi_max = 0.95;
phi_n = 41;
phi_grid = seqa( phi_min, (phi_max-phi_min)/(phi_n-1), phi_n);

/* initialize likelihood fcn
*/

prior1_grid = zeros(alp_n,rho_n);
prior2_grid = zeros(alp_n,phi_n);

like1_grid = zeros(alp_n,rho_n);
like2_grid = zeros(alp_n,phi_n);

post1_grid = zeros(alp_n,rho_n);
post2_grid = zeros(alp_n,phi_n);

/* evaluate likelihood fcns
*/

alp_ind = 1;
do until alp_ind > alp_n;

   alp = alp_grid[alp_ind];
   
   rho_ind = 1;
   do until rho_ind > rho_n;
   
      rho = rho_grid[rho_ind];
      
      like1_grid[alp_ind,rho_ind] = like1(alp,rho,rhohat,wt);
      prior1_grid[alp_ind,rho_ind] = prior1(alp,rho,wtp,-1000);
      post1_grid[alp_ind,rho_ind] = prior1(alp,rho,wtp,0)+like1(alp,rho,rhohat,wt);

      rho_ind = rho_ind+1;
   endo;
   
   alp_ind = alp_ind+1;
endo;

alp_ind = 1;
do until alp_ind > alp_n;

   alp = alp_grid[alp_ind];
   
   phi_ind = 1;
   do until phi_ind > phi_n;
   
      phi = phi_grid[phi_ind];
      
      like2_grid[alp_ind,phi_ind] = like2(alp,phi,rhohat,wt);
      prior2_grid[alp_ind,phi_ind] = prior2(alp,phi,wtp,-1000);
      post2_grid[alp_ind,phi_ind] = prior2(alp,phi,wtp,0)+like2(alp,phi,rhohat,wt);
            
      phi_ind = phi_ind+1;
   endo;
   
   alp_ind = alp_ind+1;
endo;

/* rescale prior */
prior1_mass = ln(sumc(sumc(exp(prior1_grid)))) + ln( (alp_max-alp_min)/alp_n ) 
            + ln( (rho_max-rho_min)/rho_n );

prior2_mass = ln(sumc(sumc(exp(prior2_grid)))) + ln( (alp_max-alp_min)/alp_n )
            + ln( (phi_max-phi_min)/phi_n );

prior1_grid = prior1_grid - prior1_mass;
prior2_grid = prior2_grid - prior2_mass;

post1_grid = post1_grid - prior1_mass;
post2_grid = post2_grid - prior2_mass;


/********************************************************************
* Part 5: Plot the results
********************************************************************/

plotnow:

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_pcolor = {1 4};
_plwidth = 3.5;
_plctrl = -1;
_protate = 1;
begwind;
window(3,2,1);

/*
ylabel(para_names[1]);
xlabel(para_names[2]);
*/

_pmsgstr = "Indeterminacy";
_pmsgctl = 0.25 ~ 0.3 ~ 0.5 ~ 0 ~ 1 ~ 1 ~ 0;

ytics(0,4,1,1);
xtics(0,1,0.5,1);
ztics(-40,5,5,1);
_plev =-40 | -20 | -15 | -10 | - 5| -2.5 | -0.1 ;

/******************/

title("M1 - Prior Contour");
ylabel("Alpha");
xlabel("Rho");

contour(rho_grid',alp_grid,prior1_grid); 
nextwind;

title("M2 - Prior Contour");
ylabel("Alpha");
xlabel("Phi");

contour(phi_grid',alp_grid,prior2_grid); 
nextwind;

/******************/

title("M1 - Likelihood Contour");
ylabel("Alpha");
xlabel("Rho");

contour(rho_grid',alp_grid,like1_grid); 
nextwind;

title("M2 - Likelihood Contour");
ylabel("Alpha");
xlabel("Phi");

contour(phi_grid',alp_grid,like2_grid); 
nextwind;

/******************/


title("M1 - Posterior Contour");
ylabel("Alpha");
xlabel("Rho");

contour(rho_grid',alp_grid,post1_grid); 
nextwind;

title("M2 - Posterior Contour");
ylabel("Alpha");
xlabel("Phi");

contour(phi_grid',alp_grid,post2_grid); 
endwind;


end;

/*******************************************************/

proc (1) = like1(alp,rho,rhohat,wt);
local loglh, loglhzero;

loglhzero = -50;
if alp GE 1;
   loglh = -0.5*( rho-rhohat)^2*wt;
else;
   loglh = loglhzero;
endif;

retp(loglh);
endp;


proc (1) = like2(alp,phi,rhohat,wt);
local loglh, loglhzero, rho;

loglhzero = -50;

if alp^2 < 4*phi*alp;
   loglh = loglhzero;
   goto eoflh;
endif;

if ( abs( alp - sqrt( alp^2-4*phi*alp ) ) < 2 ) AND
   ( abs( alp + sqrt( alp^2-4*phi*alp ) ) > 2 );
   rho = 0.5*( alp - sqrt( alp^2-4*phi*alp ) );
   loglh = -0.5*( rho-rhohat)^2*wt;
else;
   loglh = loglhzero;
endif;

eoflh:
retp(loglh);
endp;

proc (1) = prior1(alp,rho,wtp,logprzero);
local logpr;

if alp ge 1;
   logpr = -0.5*wtp[1]*(alp-3)^2 -0.5*wtp[2]*(rho-0.5)^2;
else;
   logpr = logprzero;
endif;
retp(logpr);
endp;

proc (1) = prior2(alp,phi,wtp,logprzero);
local logpr;

if alp^2 < 4*phi*alp;
   logpr = logprzero;
   goto eofpr;
endif;

if ( abs( alp - sqrt( alp^2-4*phi*alp ) ) < 2 ) and
   ( abs( alp + sqrt( alp^2-4*phi*alp ) ) > 2 );
   logpr = -0.5*wtp[1]*(alp-3)^2 -0.5*wtp[2]*(phi-0.5)^2;
else;
   logpr = logprzero;
endif;

eofpr:

retp(logpr);
endp;