/******************************************************************************
* NKMP_CONVCHK1.G
* plots draws from posterior distribution of parameters
* 
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include nkmp_spec.g;
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/

parasel = 1 | 4;
para_names = "Tau" $| "Psi 2";

nuse = 5000;
nblock = 100000;
nburn = 1;

/********************************************************************
* Part 1: 
********************************************************************/

mhrun   = "32";
lpath  = parapath $+ "\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\pa";
open fhdraw = ^ldraw for read;

eofloop = 0;
drawrow = seekr(fhdraw, nburn);

drawmat = readr(fhdraw,1);
drawmat = drawmat[.,parasel];
ndraws  = 1;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);

   drawmat = drawmat | drawblock[.,parasel];
   ndraws  = ndraws + nread;

   ndraws "of mhrun" $mhrun;
   pause(1e-20);

   eofloop = eof(fhdraw);

endo;

drawmat1 = drawmat;

closeall fhdraw;


/********************************************************************
* Part 2: 
********************************************************************/

mhrun   = "42";
lpath  = parapath $+ "\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\pa";
open fhdraw = ^ldraw for read;

eofloop = 0;
drawrow = seekr(fhdraw, nburn);

drawmat = readr(fhdraw,1);
drawmat = drawmat[.,parasel];
ndraws  = 1;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);

   drawmat = drawmat | drawblock[.,parasel];
   ndraws  = ndraws + nread;

   ndraws "of mhrun" $mhrun;
   pause(1e-20);

   eofloop = eof(fhdraw);

endo;

drawmat2 = drawmat;

closeall fhdraw;

/********************************************************************
* Part 3: 
********************************************************************/

mhrun   = "52";
lpath  = parapath $+ "\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\pa";
open fhdraw = ^ldraw for read;

eofloop = 0;
drawrow = seekr(fhdraw, nburn);

drawmat = readr(fhdraw, 1);
drawmat = drawmat[.,parasel];
ndraws  = 1;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);

   drawmat = drawmat | drawblock[.,parasel];
   ndraws  = ndraws + nread;

   ndraws "of mhrun" $mhrun;
   pause(1e-20);

   eofloop = eof(fhdraw);

endo;

drawmat3 = drawmat;

closeall fhdraw;

/********************************************************************
* Part 4: 
********************************************************************/

mhrun   = "62";
lpath  = parapath $+ "\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\pa";
open fhdraw = ^ldraw for read;

eofloop = 0;
drawrow = seekr(fhdraw, nburn);

drawmat = readr(fhdraw, 1);
drawmat = drawmat[.,parasel];
ndraws  = 1;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);

   drawmat = drawmat | drawblock[.,parasel];
   ndraws  = ndraws + nread;

   ndraws "of mhrun" $mhrun;
   pause(1e-20);

   eofloop = eof(fhdraw);

endo;

drawmat4 = drawmat;

closeall fhdraw;

/********************************************************
* load Posterior Mode Estimates
********************************************************/
lpara = postpath $+ "\\pm1" ;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para_mode1 = para'; 


lpara = postpath $+ "\\pm2" ;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para_mode2 = para'; 

/********************************************************************
* Part 5: Plot the results
********************************************************************/

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_pcolor = {1 4};
_plwidth = 3.5;
_plctrl = -1;
_protate = 1;
begwind;
window(3,2,0);

ylabel(para_names[1]);
xlabel(para_names[2]);

ytics(0,3,1,1);
xtics(0,2.5,0.5,1);

load path = ^postpath para2_grid, para1_grid, post1_grid, post2_grid;

ztics(-500,50,100,1);  /* m11s5t1 */
/* ztics(-300,50,25,1); */  /* m13s5t1 */


title("Posterior Contours at Low Mode"); /*low*/
_pline = ( 1 ~ 6 ~ 0 ~ para_mode2[parasel[1]] ~ 2.5 ~ para_mode2[parasel[1]] ~ 4 ~ 1) |
         ( 1 ~ 6 ~ para_mode2[parasel[2]] ~ 0 ~ para_mode2[parasel[2]] ~ 3 ~ 4 ~ 1) ;

contour(para2_grid',para1_grid,post2_grid); 
nextwind;

title("Posterior Contours at High Mode"); /* high */
_pline = ( 1 ~ 6 ~ 0 ~ para_mode1[parasel[1]] ~ 2.5 ~ para_mode1[parasel[1]] ~ 4 ~ 1) |
         ( 1 ~ 6 ~ para_mode1[parasel[2]] ~ 0 ~ para_mode1[parasel[2]] ~ 3 ~ 4 ~ 1) ;

contour(para2_grid',para1_grid,post1_grid); 
nextwind;

title("Chain 1");
_pline = ( 1 ~ 6 ~ 0 ~ para_mode2[parasel[1]] ~ 2.5 ~ para_mode2[parasel[1]] ~ 4 ~ 1) |
         ( 1 ~ 6 ~ para_mode2[parasel[2]] ~ 0 ~ para_mode2[parasel[2]] ~ 3 ~ 4 ~ 1) ;

xy(drawmat1[.,2],drawmat1[.,1]); 
nextwind;

title("Chain 2");
_pline = ( 1 ~ 6 ~ 0 ~ para_mode1[parasel[1]] ~ 2.5 ~ para_mode1[parasel[1]] ~ 4 ~ 1) |
         ( 1 ~ 6 ~ para_mode1[parasel[2]] ~ 0 ~ para_mode1[parasel[2]] ~ 3 ~ 4 ~ 1) ;

xy(drawmat2[.,2],drawmat2[.,1]); 
nextwind;

title("Chain 3");
_pline = ( 1 ~ 6 ~ 0 ~ para_mode2[parasel[1]] ~ 2.5 ~ para_mode2[parasel[1]] ~ 4 ~ 1) |
         ( 1 ~ 6 ~ para_mode2[parasel[2]] ~ 0 ~ para_mode2[parasel[2]] ~ 3 ~ 4 ~ 1) ;

xy(drawmat3[.,2],drawmat3[.,1]); 
nextwind;

title("Chain 4");
_pline = ( 1 ~ 6 ~ 0 ~ para_mode1[parasel[1]] ~ 2.5 ~ para_mode1[parasel[1]] ~ 4 ~ 1) |
         ( 1 ~ 6 ~ para_mode1[parasel[2]] ~ 0 ~ para_mode1[parasel[2]] ~ 3 ~ 4 ~ 1) ;

xy(drawmat4[.,2],drawmat4[.,1]); 
endwind;


end;
