/* filename:    paraconv1.g
** description: compare posterior means from importance sampler and MH
**              
*/

/*
goto plotnow;
*/
new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;
#include pathspec.g;

para  = pmsv(msel);
npara = rows(para);
{para,para_names} = psel(para,msel);

hmax = 140;

ndraws_grid = 1000 | seqa(100000,100000,9);
ndraws_n    = rows(ndraws_grid);

/********************************************************
* Load Posterior Mode Estimates
********************************************************/
lpara = postpath $+ "\\pm";

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para_mode = ones(ndraws_n,1).*.para;          /** paraest was saved as row vector in _pm.g **/

/******************************************************************
**         Compute Means based on MH output
******************************************************************/

mhrun       = "12";
nburn       = 1;        /* Number of initial draws to be discarded */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\pa"  ;

para_mh_mean   = zeros(ndraws_n,npara);
ndraws_gridadj = ndraws_grid - nburn+1;

open fhdraw = ^ldraw for read;

drawrow   = seekr( fhdraw, nburn);
drawblock = readr( fhdraw, ndraws_gridadj[1]);

drawsum = sumc(drawblock)';
drawsumsq = sumc(drawblock^2)';
para_mh_mean[1,.] = drawsum/ndraws_gridadj[1];

i = 2;
do until i > ndraws_n;

   "Computing MH Means, Grid Point" i "out of " ndraws_n;
   pause(1e-5);

   nread = ndraws_gridadj[i]-ndraws_gridadj[i-1];
   drawblock = readr( fhdraw, nread);
   drawsum = drawsum+sumc(drawblock)';
   drawsumsq = drawsumsq+sumc(drawblock^2)';
   para_mh_mean[i,.] = drawsum/ndraws_gridadj[i];
   i = i+1;   

endo;

drawacf0 = drawsumsq/ndraws_gridadj[ndraws_n] - para_mh_mean[ndraws_n,.]^2;
para_mh_var = drawacf0;

j = 1;
do until j > cols(drawacf0);

   "Computing MH Std, Parameter" j "out of" cols(drawacf0);
   pause(1e-5);

   drawrow = seekr( fhdraw, nburn);
   drawblock = readr( fhdraw, ndraws_gridadj[1]);
   drawcol = drawblock[.,j];

   i = 2;
   do until i > ndraws_n; 

      nread = ndraws_gridadj[i]-ndraws_gridadj[i-1];
      drawblock = readr( fhdraw, nread);
      drawcol = drawcol | drawblock[.,j];
      i = i+1;

   endo;

   l = 1;
   do until l > hmax;

      if fmod(l,10)==0;".";; pause(1e-20); endif;

      drawacf = sumc(drawcol[1+l:ndraws_gridadj[ndraws_n]].*drawcol[1:ndraws_gridadj[ndraws_n]-l])
                /ndraws_gridadj[ndraws_n] - para_mh_mean[ndraws_n,j]^2;
      para_mh_var[j] = para_mh_var[j]+2*(1-l/(hmax+1))*drawacf;
      l = l+1;

   endo;
   "";

   j = j+1;

endo;

closeall fhdraw;

para_mh_mean1 = para_mh_mean;
para_mh_std1  = sqrt(para_mh_var./ndraws_gridadj);
para_mh_low1  = para_mh_mean - 1.96*para_mh_std1;
para_mh_high1 = para_mh_mean + 1.96*para_mh_std1;


/******************************************************************
**         Compute Means based on IS output
******************************************************************/

isrun = "12";

/** input files **/
lpath = parapath $+ "\\isrun" $+ isrun;
iter  = 1;
lstat = lpath $+ "\\st" $+ ftos(iter,"%0*.*lf",2,0);
lpara = lpath $+ "\\pa" $+ ftos(iter,"%0*.*lf",2,0);

para_is_mean   = zeros(ndraws_n,npara);
para_is_std    = zeros(ndraws_n,npara);

open fhpara = ^lpara for read;
open fhstat = ^lstat for read;

j = 1;
do until j > npara;

   "Computing IS Mean/Std, Parameter" j "out of" npara;

   drawrow = seekr( fhstat,1);
   drawrow = seekr( fhpara,1);

   drawblock = readr( fhstat, ndraws_grid[1]);
   drawwgtcol = drawblock[.,4];
   nwgt = exp(drawwgtcol)/sumc(exp(drawwgtcol));

   drawblock = readr( fhpara, ndraws_grid[1]);
   drawparacol = drawblock[.,j];

   para_is_mean[1,j] = sumc( drawparacol.*nwgt )';
   para_is_std[1,j] = ndraws_grid[1]*sumc( ((drawparacol - para_is_mean[1,j])^2) .* (nwgt^2) )';
   para_is_std[1,j] = sqrt( para_is_std[1,j] ) / sqrt( ndraws_grid[1]);

   i = 2;
   do until i > ndraws_n;

      ".";; pause(1e-15);

      drawblock = readr( fhstat, ndraws_grid[i]-ndraws_grid[i-1]);
      drawwgtcol = drawwgtcol | drawblock[.,4];
      nwgt = exp(drawwgtcol)/sumc(exp(drawwgtcol));

      drawblock = readr( fhpara, ndraws_grid[i]-ndraws_grid[i-1]);
      drawparacol = drawparacol | drawblock[.,j];

      para_is_mean[i,j] = sumc( drawparacol.*nwgt )';
      para_is_std[i,j] = ndraws_grid[i]*sumc( ((drawparacol - para_is_mean[i,j])^2) .* (nwgt^2) )';
      para_is_std[i,j] = sqrt( para_is_std[i,j] ) / sqrt( ndraws_grid[i]);

      i = i+1;
   endo;
   "";

   j = j+1;

endo;

closeall fhstat, fhpara;

para_is_mean1 = para_is_mean;
para_is_low1  = para_is_mean - 1.96*para_is_std;
para_is_high1 = para_is_mean + 1.96*para_is_std;

/******************************************************************
**         Plot results
******************************************************************/

plotnow:

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {2 2 1 1 6}; 
_plwidth = 3.5;
_pcolor = { 4 4 1 1 8};
_protate = 1;

begwind;
window(4,4,0);

j = 1;
do until j > npara;

   title(para_names[j]);
   xy(ndraws_grid,para_mh_low1[.,j]~para_mh_high1[.,j]~para_is_low1[.,j]~para_is_high1[.,j]~para_mode[.,j]); 
   nextwind;

   _plegctl = { };

   j = j+1;
   
endo;

endwind;

end;
