/* filename:    mhddconv1.g
** description: compare marginal data densities: Chib vs. Geweke
**              
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;
#include pathspec.g;

ndraws_grid = 100 | seqa(10000,100000,9);
ndraws_n    = rows(ndraws_grid);

para  = pmsv(msel);
npara = rows(para);
{para,para_names} = psel(para,msel);

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

npara1  = sumc(pmaskinv);

/********************************************************
* load Posterior Mode Estimates
********************************************************/

lpost = postpath $+ "\\pmval";


/** open **/

open fhpost = ^lpost for read;
post = readr(fhpost,1);
closeall fhpost;

/******************************************************************
**         Compute Means based on MH output
******************************************************************/

mhrun       = "12";
nburn       = 1;   /* Number of initial draws to be discarded */
nblock      = 10000;

p = seqa(0.1,0.1,9);
pcrit = cdfchii(p,npara1);

geweke_dd = zeros(ndraws_n,1);

lpath = parapath $+ "\\mhrun" $+ mhrun;
lstat = lpath $+ "\\st";
lpara = lpath $+ "\\pa";

open fhstat = ^lstat for read;
open fhpara = ^lpara for read;

statrow = readr(fhstat,1);
densfac = statrow[1];

i = 1;
do until i > ndraws_n;

   /*******************************************************
   * Part 1: Compute posterior moments for parameters
   *******************************************************/

   "Part 1: Step" i "of  " ndraws_n;
   pause(1e-3);

   /** initialize **/
   ndraws  = 0;
   sumpara  = zeros(npara1,1);
   sumpara2 = zeros(npara1,npara1);

   /** discard initial draws **/
   pararow = seekr(fhpara,nburn);
   statrow = seekr(fhstat,nburn);

   eofloop = 0;
   
   do until eofloop;

     /** read block from MH output **/
     parasim = readr( fhpara, nblock );

     /** get rid of fixed parameters **/
     parasim = delif(parasim',pmask)';

     ndraws   = ndraws + rows(parasim);
   
     if ndraws GE ndraws_grid[i];
   
        parasim = parasim[1:rows(parasim)-(ndraws-ndraws_grid[i]),.];
        eofloop = 1;
      
     endif;

     sumpara  = sumpara + sumc(parasim);      /** sum **/
     sumpara2 = sumpara2 + parasim'*parasim;  /** squared (cross product) sum **/
   
   endo;

   /** posterior moments **/
   parahat       = sumpara / ndraws_grid[i];                           /** mean **/
   parasig       = sumpara2 / ndraws_grid[i] - parahat*parahat';       /** variance **/

   /** check diagonal elements **/
   j = 1;
   do until j > rows(parasig);
      if parasig[j,j] < 1E-6;
         parasig[j,j] = 1E-6;
      endif;
      j = j+1;
   endo;

   parasiginv   = inv(parasig);
   parasiglndet = ln(det(parasig));

   /*******************************************************
   * Part 2:  Compute Density estimate
   *******************************************************/

   /** initialize **/
   eofloop = 0;
   ndraws  = 0;
   suminvlike = zeros(rows(p),1);

   /** discard initial draws **/
   pararow = seekr(fhpara,nburn);
   statrow = seekr(fhstat,nburn);

   do until eofloop;

      /** read block from MH output **/
      statsim = readr( fhstat, nblock);
      parasim = readr( fhpara, nblock);

      /** get rid of fixed parameters **/
      parasim = delif(parasim',pmask)';

      ndraws   = ndraws + rows(parasim);
   
      if ndraws GE ndraws_grid[i];
   
         parasim = parasim[1:rows(parasim)-(ndraws-ndraws_grid[i]),.];
         eofloop = 1;
      
      endif;

      j = 1;
      do until j > rows(parasim);

         lnfpara = -ln(p) - 0.5*npara1*ln(2*pi) - 0.5*parasiglndet
                   -0.5*(parasim[j,.] - parahat')*parasiginv*(parasim[j,.]' - parahat);
         indpara = ((parasim[j,.] - parahat')*parasiginv*(parasim[j,.]' - parahat) .< pcrit );
         invlike = exp(lnfpara - statsim[j,1] + densfac).*indpara;

         suminvlike = suminvlike + invlike;
         j = j+1;
      endo;

   endo;

   meaninvlike = suminvlike/ndraws;
   pym3        = densfac-ln(meaninvlike);

   geweke_dd[i] = meanc(pym3);
   
   i = i+1;
   
endo;

closeall fhpara, fhstat;

/******************************************************************
**         Chib-Jeliazkov
******************************************************************/

chib_dd = zeros(ndraws_n,1);

/** open input files **/
lpath   = parapath $+ "\\mhrun" $+ mhrun;

lcjdda1   = lpath $+ "\\cjdda1";
lcjdda2q2 = lpath $+ "\\cjdda2q2";

open fhcjdda1=^lcjdda1 for read;
open fhcjdda2q2=^lcjdda2q2 for read;

block_size = 100000;

drawcola1 = 0;
eofloop = 0;

do until eofloop; 

   drawblock = readr( fhcjdda1, block_size );
   drawcola1 = drawcola1 | drawblock;
   eofloop = eof(fhcjdda1);

endo;

drawcola1 = drawcola1[2:rows(drawcola1)];

drawcola2q2 = 0;
eofloop = 0;

do until eofloop; 

   drawblock = readr( fhcjdda2q2, block_size );
   drawcola2q2 = drawcola2q2 | drawblock;
   eofloop = eof(fhcjdda2q2);

endo;

drawcola2q2 = drawcola2q2[2:rows(drawcola2q2)];

i = 1;
do until i > ndraws_n;

   "Part 2: Step" i "of  " ndraws_n;

   chib_dd[i]  = post[1] - ln( meanc( drawcola2q2[nburn:nburn+ndraws_grid[i]-1,.] )) 
                + ln(meanc(drawcola1[nburn:nburn+ndraws_grid[i]-1,.]));
   i = i+1;
endo;

/******************************************************************
**         Plot results
******************************************************************/

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_plegstr = "Modified Harm Mean\000Chib-Jeliazkov"; 
_plegctl = {1 4 48000 -194};
_plwidth = 3.5;
_pcolor = { 4 1 };
_protate = 1;

begwind;
window(2,1,0);

title("Log Marginal Data Densities");
xy(ndraws_grid,geweke_dd~chib_dd);
endwind;

end;
