/******************************************************************************
* NKMP_PREDCHK.G
* plots draws from posterior predictive distribution and the observed moment
* 
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include nkmp_spec.g;
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun   = "12";
nburn   = 1;    /* initial draws to be discarded */
nuse    = 1;

/* Data Set 1 - correct specification */

lpath  = workpath $+ "\\s5m13t1p1\\results\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\acsamp";
ldata  = lpath $+ "\\acsampdata";

open fhdraw1 = ^ldraw for read;
open fhdata1 = ^ldata for read;

drawrow = readr(fhdraw1, nburn);
drawdim = cols(drawrow);
drawrow = seekr(fhdraw1, nburn);

/* Data Set 2 - misspecification */

lpath  = workpath $+ "\\s6m13t1p1\\results\\mhrun" $+ mhrun;
ldraw  = lpath $+ "\\acsamp";
ldata  = lpath $+ "\\acsampdata";

open fhdraw2 = ^ldraw for read;
open fhdata2 = ^ldata for read;

drawrow = readr(fhdraw2, nburn);
drawdim = cols(drawrow);
drawrow = seekr(fhdraw2, nburn);

/* select two moments from the following list
** 1-3   MU_DY MU_PI MU_R
** 4-6   V_DY C0_DYPI C0_DYR
** 7-9   C0_PIDY V_PI C0_PIR
** 10-12 C0_RDY C0_RPI V_R
** 13-15 C1_DY C1_DYPI C1_DYR
** 16-18 C1_PIDY C1_PI C1_PIR
** 19-21 C1_RDY C1_RPI C1_R
*/

momsel = (5 ~ 9) |
         (8 ~ 12) |
         (16 ~ 18 ) |
         (17 ~ 21 ) ;


draw_tics = -ones(21,1) ~ ones(21,1) ~ 0.5*ones(21,1);
draw_tics[5,.] = ( 0 ~ 1 ~ 0.5);
draw_tics[9,.] = ( -0.5 ~ 1 ~ 0.5);
draw_tics[8,.] = ( 0 ~ 0.5 ~ 0.25);
draw_tics[12,.] = ( 0.5 ~ 1.5 ~ 0.5);
draw_tics[16,.] = ( -0.5 ~ 0.5 ~ 0.5);
draw_tics[18,.] = ( -0.5 ~ 1 ~ 0.25);
draw_tics[17,.] = ( 0 ~ 1 ~ 0.5);
draw_tics[21,.] = ( 0 ~ 1 ~ 0.5);




draw_names = "Mean (Delta Y)" $|
             "Mean (Pi)" $|
             "Mean (R)" $|
             "Std (Delta Y)" $|
             "Corr (Delta Y, Pi)" $|
             "Corr (Delta Y, R)" $|             
             "Corr (Delta Y, Pi)" $|
             "Std (Pi)" $|
             "Corr (R, Pi)" $|
             "Corr (Delta Y, R)" $|             
             "Corr (R, Pi)" $|
             "Std (R)" $|
             "Corr (Delta Y, Delta Y(-1))" $|
             "Corr (Delta Y, Pi(-1))" $|
             "Corr (Delta Y, R(-1))" $|
             "Corr (Pi, Delta Y(-1))" $|
             "Corr (Pi, Pi(-1))" $|
             "Corr (Pi, R(-1))" $|
             "Corr (R, Delta Y(-1))" $|
             "Corr (R, Pi(-1))" $|
             "Corr (R, R(-1))";

         
/********************************************************************
* Part 1: load actual sample moments
********************************************************************/

sampmomdata1  = readr(fhdata1,1);
sampmomdata2  = readr(fhdata2,1);

/********************************************************************
* Part 2: load draws from posterior predictive distribution
********************************************************************/

/* Data Set 1: Correct Specification */

drawmat    = zeros(1,drawdim);

eofloop = 0;
nblock  = 1000;
ndraws  = 0;

do until eofloop;

   drawblock  = readr(fhdraw1,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);

   drawmat = drawmat | drawblock;
   ndraws  = ndraws + nread;

   ".";;pause(1e-20);

   eofloop = eof(fhdraw1);

endo;

drawmat1 = drawmat[2:ndraws+1,.];

/* Data Set 2: Misspecification */

drawmat    = zeros(1,drawdim);

eofloop = 0;
nblock  = 1000;
ndraws  = 0;

do until eofloop;

   drawblock  = readr(fhdraw2,nblock);
   nread   = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   drawblock = delif(drawblock,indseq);
   nread   = rows(drawblock);

   drawmat = drawmat | drawblock;
   ndraws  = ndraws + nread;

   ".";;pause(1e-20);

   eofloop = eof(fhdraw2);

endo;

drawmat2 = drawmat[2:ndraws+1,.];

closeall fhdraw1, fhdata1, fhdraw2, fhdata2;

/********************************************************************
* Part 3: Plot the results
********************************************************************/

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_pcolor = 1;
_plwidth = 3.5;
_plctrl = -1;
_protate = 1;
begwind;
window(4,2,0);

i = 1;
do until i > 4;

   xlabel(draw_names[momsel[i,1]]);
   ylabel(draw_names[momsel[i,2]]);

   /* Data Set 1 */

   _pline = ( 1 ~ 6 ~ draw_tics[momsel[i,1],1] ~ sampmomdata1[momsel[i,2]] ~ draw_tics[momsel[i,1],2] ~ sampmomdata1[momsel[i,2]] ~ 4 ~ 1) |
            ( 1 ~ 6 ~ sampmomdata1[momsel[i,1]] ~ draw_tics[momsel[i,2],1] ~ sampmomdata1[momsel[i,1]] ~ draw_tics[momsel[i,2],2] ~ 4 ~ 1) ;
 
   xtics(draw_tics[momsel[i,1],1],draw_tics[momsel[i,1],2],draw_tics[momsel[i,1],3],1);
   ytics(draw_tics[momsel[i,2],1],draw_tics[momsel[i,2],2],draw_tics[momsel[i,2],3],1);

   xy(drawmat1[.,momsel[i,1]],drawmat1[.,momsel[i,2]]);
   nextwind;

   /* Data Set 2 */  

   _pline = ( 1 ~ 6 ~ draw_tics[momsel[i,1],1] ~ sampmomdata2[momsel[i,2]] ~ draw_tics[momsel[i,1],2] ~ sampmomdata2[momsel[i,2]] ~ 4 ~ 1) |
            ( 1 ~ 6 ~ sampmomdata2[momsel[i,1]] ~ draw_tics[momsel[i,2],1] ~ sampmomdata2[momsel[i,1]] ~ draw_tics[momsel[i,2],2] ~ 4 ~ 1) ;
 
   xtics(draw_tics[momsel[i,1],1],draw_tics[momsel[i,1],2],draw_tics[momsel[i,1],3],1);
   ytics(draw_tics[momsel[i,2],1],draw_tics[momsel[i,2],2],draw_tics[momsel[i,2],3],1);

   xy(drawmat2[.,momsel[i,1]],drawmat2[.,momsel[i,2]]);
   nextwind;

   i = i+1;

endo;

endwind;

end;
