/******************************************************************************
* fig_prior-posterior-draws.g
* plots bivariate scatters
* 
*
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;
#include pathspec.g;


/********************************************************************
* Load Parameter Draws -- Posterior
********************************************************************/
mhrun  = "12";
nburn  = 10001;
nblock = 10000;    /* block size */
nuse   = 5000;    /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

drawrow = seekr(fhpara,nburn);
drawblock = readr(fhpara,1);

loopct = 1;
eofloop = 0;

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);


   if ismiss(parasim) == 0;
 
      drawblock = drawblock | parasim;
      
   endif;
   
   if _verbose;
      "";
      "Draws Processed " ftos(rows(parasim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara;

posteriordraws = drawblock;

/********************************************************************
* Load Parameter Draws -- Prior
********************************************************************/
mhrun  = "0";
nburn  = 1;
nblock = 10000;    /* block size */
nuse   = 500;     /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

drawrow = seekr(fhpara,nburn);
drawblock = readr(fhpara,1);

loopct = 1;
eofloop = 0;

do until eofloop;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);

   if ismiss(parasim) == 0;
 
      drawblock = drawblock | parasim;
      
   endif;
   
   if _verbose;
      "";
      "Draws Processed " ftos(rows(parasim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara;

priordraws = drawblock;

/********************************************************
* load Posterior Mode Estimates
********************************************************/
lpara = postpath $+ "\\pm" ;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para_mode = para'; 


/********************************************************************
* Create Plots
********************************************************************/

para_names = "Tau" $|
             "Kappa" $|
             "Psi 1" $|
             "Psi 2" $|
             "Rho R" $|
             "Rho G" $|
             "Rho Z" $|
             "RRst " $|
             "PIst " $|
             "GAMst" $|
             "Sig R" $|
             "Sig G" $|
             "Sig Z" ;

para_indvec = (1 ~ 2) |
              (3 ~ 4) |
              (5 ~ 11) |
              (6 ~ 7) |
              (9 ~ 10) |
              (12 ~ 13) ;

para_tics = (1 ~ 4 ~ 1) |
            (0 ~ 0.6 ~ 0.2) |
            (1 ~ 3 ~ 0.5) |
            (0 ~ 2.5 ~ 0.5) |
            (0 ~ 1 ~ 0.25 ) |
            (0 ~ 1 ~ 0.25 ) |
            (0 ~ 1 ~ 0.25 ) |
            (0 ~ 1 ~ 0.25 ) |
            (2 ~ 6 ~ 1    ) |
            (0 ~ 1 ~ 0.25 ) |
            (0 ~ 1 ~ 0.25 ) |
            (0 ~ 1 ~ 0.25 ) |
            (0 ~ 1 ~ 0.25 ) ;
         

graphset;
begwind;
margin(0.2,0.2,0.2,0.2);
window(7,2,0);
fonts("microb");
_ptitlht = 0.6;
_paxht = 0.7;
_pnumht = 0.7;
_pcolor = {1 4 4};
_pltype = {6 2 2};
_protate = 1;
_plctrl = -1;
_plwidth = 5.5;
_psymsiz = 5;

nextwind;
nextwind;

i = 1;
do until i > 6;

  if i == 1;
      _pmsgstr = "Prior";
      _pmsgctl = ( 3 ~ 10 ~ 0.9 ~ 0 ~ 2 ~ 0 ~ 0.7);
  else;
     _pmsgctl = 0;
  endif;

  _pline = ( 1 ~ 6 ~ para_tics[para_indvec[i,1],1] ~ para_mode[para_indvec[i,2]] ~ para_tics[para_indvec[i,1],2] ~ para_mode[para_indvec[i,2]] ~ 4 ~ 1) |
           ( 1 ~ 6 ~ para_mode[para_indvec[i,1]] ~ para_tics[para_indvec[i,2],1] ~ para_mode[para_indvec[i,1]] ~ para_tics[para_indvec[i,2],2] ~ 4 ~ 1) ;

  xtics(para_tics[para_indvec[i,1],1],para_tics[para_indvec[i,1],2],para_tics[para_indvec[i,1],3],1);
  ytics(para_tics[para_indvec[i,2],1],para_tics[para_indvec[i,2],2],para_tics[para_indvec[i,2],3],1);

  xlabel(para_names[para_indvec[i,1]]);
  ylabel(para_names[para_indvec[i,2]]);

  xy(priordraws[.,para_indvec[i,1]],priordraws[.,para_indvec[i,2]]);
  nextwind;

  if i == 1;
      _pmsgstr = "Posterior";
      _pmsgctl = ( 3 ~ 10 ~ 0.9 ~ 0 ~ 2 ~ 0 ~ 0.7);
  else;
     _pmsgctl = 0;
  endif;

  xy(posteriordraws[.,para_indvec[i,1]],posteriordraws[.,para_indvec[i,2]]);
  nextwind;

  i = i+1;

endo;

endwind;
         
end;
