/******************************************************************/
/*                                                                */
/*              Plot the variance decomposition                   */
/*                                                                */
/*                        Frank Schorfheide                       */
/*                                                                */
/******************************************************************/

/* filename: vdplot.g
** created:  09/27/01
*/

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include nkmp_spec.g;
#include pathspec.g;



/* Configure the Plots
**
**
*/

mhrun  = "0" $| "12";
nblock = 1000;

graphset;
begwind;

makewind(2.5, 2.5, 0.5, 3.0, 0);
makewind(2.5, 2.5, 3.5, 3.0, 0);
makewind(2.5, 2.5, 0.5, 0, 0);
makewind(2.5, 2.5, 3.5, 0, 0);
fonts("microb");
_pagesiz = {6,6};

_ptitlht = 0.5;
_paxht = 0.3;
_pnumht = 0.25;
_pcolor =2 ;
_pltype = {6 1 2 2};
/*_protate = 1; */
_plctrl = -1;
_plwidth = 5.5;
_psymsiz = 2;
_paxes = 0;

eps1 = 0.1;
xtics(-1,1,0.1,1);
ytics(-0.3,1.5,0.1,1);

varind = 1;
do until varind > 2;

   mhind = 1;
   do until mhind > 2;

      lpath = resupath $+ "\\mhrun" $+ mhrun[mhind];

      /* Load draws from posterior
      */ 
      ldraws = lpath $+ "\\vdsimp";

      open fhdraws = ^ldraws for read;
      drawblock = readr(fhdraws,1);
      vdsimplexsim = drawblock[.,(varind-1)*2+1:(varind-1)*2+2];
      
      eofloop = 0;

      do until eofloop;

         drawblock  = readr(fhdraws,nblock);
         vdsimplexsim = vdsimplexsim | drawblock[.,(varind-1)*2+1:(varind-1)*2+2];
         ".";;pause(1e-20);
   
         eofloop = eof(fhdraws);

      endo;
 
      closeall fhdraws;

     
      if (mhind == 1) AND (varind == 1);
         titlstr = "Prior - Output Growth";
      elseif (mhind == 2) AND (varind == 1);
         titlstr = "Posterior - Output Growth";
      elseif (mhind == 1) AND (varind == 2);
         titlstr = "Prior - Inflation";
      else;
         titlstr = "Posterior - Inflation";
      endif;

      title(titlstr);

      _pmsgstr = "R\000G\000Z";
      _pmsgctl =  (-1/sqrt(2) - eps1 ~ 0 -eps1 ~ 0.4 ~ 0 ~ 1 ~ 4 ~ 1)|
                  ( 1/sqrt(2) + eps1 ~ 0 -eps1 ~ 0.4 ~ 0 ~ 1 ~ 4 ~ 1)|
                  ( 0 ~ sqrt(3/2)+eps1        ~ 0.4 ~ 0 ~ 1 ~ 4 ~ 1);

      _pline = ( 1 ~ 1 ~ -1/sqrt(2) ~ 0 ~ 1/sqrt(2) ~ 0 ~ 4 ~ 1 )|
               ( 1 ~ 1 ~ -1/sqrt(2) ~ 0 ~ 0 ~ sqrt(1.5) ~ 4 ~ 1 )|
               ( 1 ~ 1 ~  1/sqrt(2) ~ 0 ~ 0 ~ sqrt(1.5) ~ 4 ~ 1 );

      xy(vdsimplexsim[.,1],vdsimplexsim[.,2]);
      nextwind;
   
      mhind=mhind+1;
   endo;
   
   varind = varind + 1;

endo;
endwind;
