/********************************************************
* HEAD.G
* upadted:  06/23/2005
********************************************************/

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs  = rows(YY);          /** number of observations **/

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/** parameter specification: transformation and bounds **/
_trspec      = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;
_bounds      = _trspec[.,2 3];


/********************************************************
* Load Posterior Mode Estimates
********************************************************/
lpara = postpath $+ "\\pm" $+ pmsel;
lretc = postpath $+ "\\pmret" $+ pmsel;
lpost = postpath $+ "\\pmval" $+ pmsel;

/** check posterior mode results **/
if not(exist(lpara $+ ".dat"));
    "";
    "Please run posterior maximization program first.";
    "";
    end;
endif;

/** open **/
open fhpara = ^lpara for read;
para = readr(fhpara,1);
closeall fhpara;

para  = para';          /** paraest was saved as row vector in _pm.g **/
npara = rows(para);

open fhpost = ^lpost for read;
post = readr(fhpost,1);
closeall fhpost;

open fhretc = ^lretc for read;
_csretcode = readr(fhretc,1);
closeall fhretc;


/********************************************************
* Load multiplication matrix for parameters
********************************************************/
lmult = postpath $+ "\\mult" $+ pmsel;       /** sigmult **/
lhess = postpath $+ "\\hess" $+ pmsel;     /** HHM **/


/** check Hessian results **/
if not(exist(lmult $+ ".dat"));
    "";
    "Please run Hessian finding program first.";
    "";
    end;
endif;

/** open **/
open fhmult = ^lmult for read;
SIGSCALE    = readr( fhmult,npara );
closeall fhmult;

open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

/** rank of Hessian (theoretical) **/
SIGPROPdim = sumc(pmaskinv);

/** inverse by Singular Value Decomposition (SVD) **/
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

/** log determinant for nonsingular part **/
i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;
