/******************************************************************************
* NKMP_AC.G
* converts the DSGE parameters into unconditional moments
* variance decomposition and autocovariances
*
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;
#include pathspec.g;


/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun  = "12";
nblock = 10000;    /* block size */
nuse   = 5000;     /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

/** check dimensions **/
para    = pmsv(msel);
{vd,ac} = dsgemom(para,msel);

nac = cols(ac);
nvd = rows(vd)*cols(vd);

/** Initialize Output files **/
oac  = opath $+ "\\ac";
ovd  = opath $+ "\\vd";

create fhac = ^oac with AC, nac, 8;
create fhvd = ^ovd with VD, nvd, 8;

/** Initialization **/
eofloop = 0;
loopct  = 1;

if _verbose;
   "";
   "Calculating moments...";
   "----------------------";
endif;

do until eofloop;

   if _verbose;
      "";
      "Block           " ftos(loopct,"%*.*lf",6,0);
   endif;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

if ismiss(parasim) == 0;

   acsim   = zeros(nread,nac);
   vdsim   = zeros(nread,nvd);

   j = 1;
   do until j > nread;
      para = parasim[j,.]';
      
      {vd,ac} = dsgemom(para,msel);

      /* variance Decompositions */
      vdsim[j,.] =  vec(vd')';

      /* autocovariances */
      acsim[j,.] = ac;

      if _verbose;
         if fmod(j,10)==0; ".";; pause(1e-20);endif;
      endif;
      j = j+1;
   endo;

    /** write Output **/
   wr = writer(fhac,acsim);
   wr = writer(fhvd,vdsim);

endif;

   if _verbose;
      "";
      "Draws Processed " ftos(rows(acsim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara, fhac, fhvd;

end;


/********************************************************************
* Procedure: DSGEMOM
********************************************************************/
proc (2) = dsgemom(para,msel);

local TT,QQ,RR,HH,DD,ZZ,VV,RC,sig_chol,i,ny,
      GAM0s,GAM1s,GAM0i,GAM1i,GAM0,GAM1,QQi,GAM0tr,seldiag,seloffdiag,
      vd,ac;

/**********************************************************
**      System matrices for State-space
**********************************************************/
{TT,QQ,RR,HH,DD,ZZ,VV,RC}=sysmat(para,msel);
ny = rows(ZZ);

/**********************************************************
**   variance decomposition and autocovariance
**********************************************************/

/** sqrt decomposition **/
sig_chol = sqrt(QQ);

GAM0 = zeros(ny,ny);
GAM1 = zeros(ny,ny);
vd   = zeros(ny,cols(sig_chol));

i=1;
do until i > cols(sig_chol);

   QQi   = sig_chol[.,i]*sig_chol[.,i]';
   GAM0s = real(dlyap(TT,RR*QQi*RR'));
   GAM0i = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
   GAM1i = ZZ*(TT*GAM0s)*ZZ' + ZZ*(TT*RR*VV);

   GAM0 = GAM0 + GAM0i;
   GAM1 = GAM1 + GAM1i;

   vd[.,i] = diag(GAM0i);

   i = i+1;
endo;

/** variance decomposition **/
vd = vd./diag(GAM0);

/** autocovariance **/

GAM0tr     = sqrt(diag(GAM0))*( sqrt(diag(GAM0)) )';
seldiag    = eye(rows(GAM0));
seloffdiag = ones(rows(GAM0),cols(GAM0)) - seldiag;
      
GAM0   = seldiag.*sqrt(GAM0tr) + seloffdiag.*( GAM0./GAM0tr);
GAM1   = GAM1./GAM0tr;

/** ny + ny^2 + ny^2 **/
ac = DD' ~ vec(GAM0)' ~ vec(GAM1)';

retp(vd,ac);
endp;
