/******************************************************************************
* NKMP_ACSAMP.G
* converts the DSGE parameters into unconditional sample moments
* 
*
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

mhrun  = "12";

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;
#include pathspec.g;
#include nkmp_sim.src;

/********************************************************
* Import data series
********************************************************/
#include loaddata.g;

nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

nobs_burn = 100;
nobs_total = nobs_burn + nobs;

YYmean = meanc(YY);

GAM0 = YY'*YY/nobs - YYmean*YYmean';
/* GAM1 = YY[2:nobs,.]'*YY[1:nobs-1,.]/nobs - YYmean*YYmean';  */
GAM1 = ( YY[2:nobs,.]-( meanc(YY[2:nobs,.]))' )'
      *( YY[1:nobs-1,.] - (meanc(YY[1:nobs-1,.]))'  )/(nobs-1);

/** autocovariance **/

GAM0tr     = sqrt(diag(GAM0))*( sqrt(diag(GAM0)) )';
seldiag    = eye(rows(GAM0));
seloffdiag = ones(rows(GAM0),cols(GAM0)) - seldiag;
      
GAM0   = seldiag.*sqrt(GAM0tr) + seloffdiag.*( GAM0./GAM0tr);
GAM1   = GAM1./GAM0tr;

/** ny + ny^2 + ny^2 **/
ac = YYmean' ~ vec(GAM0)' ~ vec(GAM1)';
nac = cols(ac);

/** Initialize Output files **/
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir=chk_dir(opath);
oac  = opath $+ "\\acsampdata";
create fhac = ^oac with AC, nac, 8;
wr = writer(fhac,ac);
closeall fhac;


/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
nblock = 10000;   /* block size */
nuse   = 5000;    /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara  = lpath $+ "\\pa";

open fhpara = ^lpara for read;

/** check dimensions **/
para    = pmsv(msel);
ac      = dsgesampmom(para,msel,nobs_burn,nobs_total);

nac = cols(ac);


/** Initialize Output files **/
oac  = opath $+ "\\acsamp";
create fhac = ^oac with AC, nac, 8;


/** Initialization **/
eofloop = 0;
loopct  = 1;

if _verbose;
   "";
   "Calculating moments...";
   "----------------------";
endif;

do until eofloop;

   if _verbose;
      "";
      "Block           " ftos(loopct,"%*.*lf",6,0);
   endif;

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   parasim = delif(parasim,indseq);
   nread   = rows(parasim);

if ismiss(parasim) == 0;

   acsim   = zeros(nread,nac);

   j = 1;
   do until j > nread;
      para = parasim[j,.]';
      
      ac = dsgesampmom(para,msel,nobs_burn,nobs_total);

      /* autocovariances */
      acsim[j,.] = ac;

      if _verbose;
         if fmod(j,10)==0; ".";; pause(1e-20);endif;
      endif;
      j = j+1;
   endo;

    /** write Output **/
   wr = writer(fhac,acsim);

endif;

   if _verbose;
      "";
      "Draws Processed " ftos(rows(acsim),"%*.*lf",6,0);
        "";
   endif;

   loopct = loopct + 1;
   eofloop = eof(fhpara);

endo;

closeall fhpara, fhac;

end;


/********************************************************************
* Procedure: DSGESAMPMOM
********************************************************************/
proc (1) = dsgesampmom(para,msel,nobs_burn,nobs_total);

local YYsim, YYmean, YYnobs, GAM0, GAM1, GAM0tr, seldiag, seloffdiag;

/**********************************************************
**      simulate date
**********************************************************/
YYsim = dsgesim(para,msel,nobs_burn,nobs_total);

YYmean = meanc(YYsim);
YYnobs = rows(YYsim);

GAM0 = YYsim'*YYsim/YYnobs - YYmean*YYmean';
/* GAM1 = YYsim[2:YYnobs,.]'*YYsim[1:YYnobs-1,.]/YYnobs - YYmean*YYmean'; */
GAM1 = ( YYsim[2:YYnobs,.]-( meanc(YYsim[2:YYnobs,.]))' )'
      *( YYsim[1:YYnobs-1,.] - (meanc(YYsim[1:YYnobs-1,.]))'  )/(YYnobs-1);

/** autocovariance **/

GAM0tr     = sqrt(diag(GAM0))*( sqrt(diag(GAM0)) )';
seldiag    = eye(rows(GAM0));
seloffdiag = ones(rows(GAM0),cols(GAM0)) - seldiag;
      
GAM0   = seldiag.*sqrt(GAM0tr) + seloffdiag.*( GAM0./GAM0tr);
GAM1   = GAM1./GAM0tr;

if minc(minc(GAM1)) < - 1;
  GAM0;
  GAM1;
  GAM0tr;
  YYmean;
  YYnobs;
  end;
endif;

/** ny + ny^2 + ny^2 **/
ac = YYmean' ~ vec(GAM0)' ~ vec(GAM1)';

retp(ac);
endp;
