/* filename:    nkpm_pri1.g
** description: The program generates draws from the prior of the 
**              DSGE model
** created:     
*/
new;

/** set seed **/
/** random number generator used in optimization procedure **/
/* rndseed 1234; */

/********************************************************
* House cleaning and set defaults
********************************************************/
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;

#include pathspec.g;

block_n    = 10;
block_size = 10000;
mhrun      = "0";   /* NOTE: we use mhrun = 0 to denote prior */
opath      = parapath $+ "\\mhrun" $+ mhrun;
chkdir  = chk_dir(opath);

para  = pmsv(msel);
npara = rows(para);

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/******************************************************** 
**  Initialize Output files
*/

opath   = parapath $+ "\\mhrun" $+ mhrun;
chkdir  = chk_dir(opath);

otheta = opath $+ "\\pa0";
create fhtheta = ^otheta with THETA, npara, 8;

/* Loop to generate parameter draws
*/
block_it = 1;
para_acpt = 0;

do until block_it > block_n;

   /* Generate THETA draws
   */

   parasim = zeros(block_size,npara);

   i = 1; 
   do until i > npara;

     if pmask[i] == 1;

      parasim[.,i] = ones(block_size,1)*pfix[i];

     else;
   
      if pshape[i] == 1; /* BETA Prior */
         a = (1-pmean[i])*pmean[i]^2/pstdd[i]^2 - pmean[i];
         b = a*(1/pmean[i] - 1);
         parasim[.,i] = rndbeta(block_size,1,a,b);
      
      elseif pshape[i] == 2; /* GAMMA PRIOR */
         b = pstdd[i]^2/pmean[i];
         a = pmean[i]/b;
         parasim[.,i] = b*rndgam(block_size,1,a);
      
      elseif pshape[i] == 3; /* GAUSSIAN PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = a + b*rndn(block_size,1);

      elseif pshape[i] == 4; /* INVGAMMA PRIOR */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = sqrt( b*a^2/sumc( (rndn(b,block_size))^2 ) );

      elseif pshape[i] == 5; /* uniform prior */
         a = pmean[i];
         b = pstdd[i];
         parasim[.,i] = a + (b-a)*rndu(block_size,1);

      elseif pshape[i] == 0; /* No Prior - Fixed */
         a = pmean[i];
         parasim[.,i] = a*ones(block_size,1);

      endif;

     endif;    

     i = i+1;
   endo; 

   "Processing Block" block_it;
   
   wr = writer(fhtheta,parasim);

   block_it = block_it + 1;

endo;

closeall fhtheta;
end;
