/* filename:    paratab0.g
** description: generates a LaTeX Table  
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include nkmp_spec.g;
#include pathspec.g;
#include nkmp_para.src;

para  = pmsv(msel);
npara = rows(para);
{para,para_names} = psel(para,msel);

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];

/* simulation parameters */
load path = ^priopath para[npara,3] = "msim_par.out";
parasim   = para[.,1];

/******************************************************************
**         Load Parameter Draws from MH Output
*/

/* Specify input files
** filename: lmodel + lprior + dataselstr + subTstr + pas
*/
inputfile_str = "\\mhrun0\\pas";

/* Specify row names for table
*/
row_name =  "$\\tau$          &  $\\RR^+$  & Gamma  "
         $| "$\\kappa$        &  $\\RR^+$  & Gamma  "
         $| "$\\psi_1$        & $\\RR^+$ & Gamma "
         $| "$\\psi_2$        & $\\RR^+$ & Gamma "
         $| "$\\rho_R$        & $[0,1)$  & Beta  "
         $| "$\\rho_G$        & $[0,1)$  & Beta  "
         $| "$\\rho_Z$        & $[0,1)$  & Beta  "
         $| "$r^{(A)}$        & $\\RR^+$ & Gamma "
         $| "$\\pi^{(A)}$     & $\\RR^+$ & Gamma "
         $| "$\\gamma^{(Q)}$  & $\\RR$   & Normal"
         $| "$\\sigma_R$      & $\\RR^+$ & InvGamma "
         $| "$\\sigma_G$      & $\\RR^+$ & InvGamma "
         $| "$\\sigma_Z$      & $\\RR^+$ & InvGamma ";

             
inputfile_n = rows(inputfile_str);

post_mean = zeros(npara,inputfile_n);
post_stdd = zeros(npara,inputfile_n);
post_ci   = zeros(npara,inputfile_n*2);


/* read the input files
*/
inputfile_ind = 1;

do until inputfile_ind > inputfile_n;

   lparas = parapath $+ inputfile_str[inputfile_ind];
   open fhparas = ^lparas for read;
   
   post_mean[.,inputfile_ind] = readr(fhparas,1)';
   post_stdd[.,inputfile_ind] = readr(fhparas,1)';
   drawrow = seekr(fhparas,npara+3);
   post_ci[.,2*(inputfile_ind-1)+1:2*inputfile_ind] = readr(fhparas,2)';

   inputfile_ind = inputfile_ind+1;
   
   closeall fhparas;
   
endo;

/* Create table
*/
para_ind = 1;
do until para_ind > npara;

   tablerow = row_name[para_ind];

   inputfile_ind = 1;
   do until inputfile_ind > inputfile_n;
      
      tablerow = tablerow 
                 $+ " & " $+ ftos(pmean[para_ind],"%*.*lf",5,2)
                 $+ " & " $+ ftos(pstdd[para_ind,inputfile_ind],"%*.*lf",5,2)
                 $+ " & " $+ ftos(parasim[para_ind],"%*.*lf",5,2);
                /*  $+ " & " $+ ftos(post_mean[para_ind,inputfile_ind],"%*.*lf",5,2)
                 $+ " & [" $+ ftos(post_ci[para_ind,2*(inputfile_ind-1)+1],"%*.*lf",5,2)
                 $+ ", "  $+ ftos(post_ci[para_ind,2*(inputfile_ind)],"%*.*lf",5,2) $+ "]"; */
      inputfile_ind = inputfile_ind+1;
      
   endo;
   tablerow  = tablerow  $+ "\\\\";
   
   $tablerow;
   
   para_ind = para_ind+1;

endo;
