/******************************************************************************
* VDCONV.G
* converts variance decomposition into simplex coordinates
* 
*
* upadted:  03/11/2005
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  _verbose = 1;           /** 1 for verbose **/

#include nkmp_spec.g;
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun   = "12";
nburn   = 1;    /* initial draws to be discarded */
nuse    = 1;

lpath  = resupath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;

ldraw  = lpath $+ "\\vd";
odraws = opath $+ "\\vdsimp";

open fhdraw = ^ldraw for read;

create fhdraws = ^odraws with VDSIMP, 6, 8;

drawrow = readr(fhdraw, nburn);
drawdim = cols(drawrow);
drawrow = seekr(fhdraw, nburn);

/********************************************************************
* Part 1: convert the variance decomposition into simplex coordinates
********************************************************************/

eofloop = 0;
nblock  = 10000;

if _verbose;
   "";
   "=============================================";
   "Forecasting Error Variance Decomposition     ";
   "";
   "Model            " $lmodel;
   "Data set         " $dataselstr;
   "Prior            " $lprior;
   "Subsample        " $subTstr;
   "MHrun Index      " $mhrun;
   "=============================================";
   "";
   "Conversion into Simplex Coordinates";
endif;

do until eofloop;

   drawblock  = readr(fhdraw,nblock);
   ndraws = rows(drawblock);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,ndraws);
   indseq  = indseq % nuse;
   indseq  = indseq .NE 0;

   drawblock = delif(drawblock,indseq);
   ndraws = rows(drawblock);

   /*
   draw_names = {
                 DY_R DY_G DY_Z
                 PI_R PI_G PI_Z
                 R_R  R_G  R_Z 
                };
   */

   vdsimplexsim = zeros(ndraws,6);
   
   j = 1;
   do until j > 3;
   
      /* iterate over variables */
      
      /* Convert coordinates
      */
      ysim = sqrt(3/2)*drawblock[.,(j-1)*3+3]; 
      xsim = ( sqrt(2)*drawblock[.,(j-1)*3+2]./ (drawblock[.,(j-1)*3+2]+drawblock[.,(j-1)*3+1]) 
              - 1/sqrt(2) ) .*(sqrt(3/2)-ysim)/sqrt(3/2);
              
      vdsimplexsim[.,(j-1)*2+1] = xsim;
      vdsimplexsim[.,(j-1)*2+2] = ysim;

      j = j+1;
      
   endo;

   if _verbose;
      ".";;pause(1e-20);
   endif;

   wr = writer(fhdraws, vdsimplexsim );

   eofloop = eof(fhdraw);

endo;

closeall fhdraw, fhdraws;

end;
